/* $Revision: 1.4 $ */ 

/******************************************************************************
 *
 * Copyright 2012 NetBurner  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or its derivatives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non-NetBurner Hardware.
 *   Please contact sales@Netburner.com for more information.
 *
 *   NetBurner makes no representation or warranties
 *   with respect to the performance of this computer program, and
 *   specifically disclaims any responsibility for any damages,
 *   special or consequential, connected with the use of this program.
 *
 *---------------------------------------------------------------------
 * NetBurner
 * 5405 Morehouse Dr.
 * San Diego, Ca. 92121
 *
 * information available at:  http://www.netburner.com
 * E-Mail info@netburner.com
 *
 * Support is available: E-Mail support@netburner.com
 *
 *****************************************************************************/


#include <basictypes.h>
#include <ucos.h>
#include <ucosmcfc.h>
#include <constants.h>
#include <string.h>
#include <cfinter.h>
#include <sim.h>
#include <stdio.h>
#include "HiResTimer.h"

#ifdef MOD5270B
#ifndef MOD5270
#define MOD5270
#endif
#endif

#if ( defined MOD5441X  )
#define sim sim2
#endif

#ifdef MOD5213
#define FAST_USER_VAR
#define ter dter
#define tmr dtmr
#define trr dtrr
#define tcn dtcn
#define txmr dtxmr
#endif

#if( defined MOD5270 || defined MOD5282 || defined MOD5234 || defined MOD5213 || defined MOD5441X  )
#include <pins.h>
#endif

#if defined MCF5272
#error HiResTimer is not compatible with, nor supported on the MCF5272 processor.
#endif

/* An interrupt setup helper defined in bsp.c */
extern "C"
{
#if( defined MCF5270 || defined MCF5213 )
/* This function sets up the 5270 interrupt controller */
void SetIntc(long func, int vector, int level, int prio );
#elif( defined MCF52234 )
void SetIntc0( long func, int vector, int level, int prio );
#elif( defined MCF5282 || defined MCF5234 )
void SetIntc(int intc, long func, int vector, int level, int prio );
#elif ( defined MOD5441X  )
void SetIntc(int intcnum,  long func, int vector, int level);
#else
#error PROCESSOR NOT DEFINED
#endif
}

#ifndef FAST_USER_VAR
#define FAST_USER_VAR 
#endif

HiResTimer HiResTimer::timers[TIMER_COUNT] FAST_USER_VAR;
BYTE HiResTimer::timerCreated[TIMER_COUNT] FAST_USER_VAR;

INTERRUPT(Timer0_Interrupt, 0x2700 )
{
    HiResTimer::fireInterrupt( 0 );
}
INTERRUPT(Timer1_Interrupt, 0x2700 )
{
    HiResTimer::fireInterrupt( 1 );
}
INTERRUPT(Timer2_Interrupt, 0x2700 )
{
    HiResTimer::fireInterrupt( 2 );
}
INTERRUPT(Timer3_Interrupt, 0x2700 )
{
    HiResTimer::fireInterrupt( 3 );
}

// Call on the appropriate timer objects.
inline void HiResTimer::fireInterrupt( int timer )
{
   HiResTimer::timers[timer].fireInterrupt();
}

// Interrupt routine to handle timer resets.
inline void HiResTimer::fireInterrupt()
{
   simTimer_->ter = 0x2;
   resetCount_++;
   if (interruptFunction_)
      interruptFunction_();
   if (delayCalled_)
   {
      OSSemPost(&delaySem_);
      stopClear();
      delayCalled_ = FALSE;
   }
}


/*
 * Performs the expensive (double math) tasks for setting up a high resolution time.
 * Inputs:
 *      InterruptTime - delayTime between interrupts
 */
void HiResTimer::init(double InterruptTime)
{
   stopClear();
   DWORD ReferenceTicks = 0xFFFFFFFF;
   if (InterruptTime != 0)
   {
      prescaler_ = (InterruptTime * (double)NB_CPU_CLK) / (double) ReferenceTicks;
      InterruptTime /= (double) (prescaler_ + 1.0);
      ReferenceTicks = (InterruptTime * (double) NB_CPU_CLK);
   }

   simTimer_->txmr = 0x00;
   simTimer_->tmr = (0x19 | ((WORD) prescaler_ << 8)); // Enable
   simTimer_->trr = ReferenceTicks;

   delayCalled_ = FALSE;

   if (firstInit_ == FALSE)
   {
      OSSemInit(&delaySem_, 0);
#if( defined MCF5270 || defined MCF5213 )
      if( timer_ == 0 )
          SetIntc((long)&Timer0_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 1 )
          SetIntc((long)&Timer1_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 2 )
          SetIntc((long)&Timer2_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 3 )
          SetIntc((long)&Timer3_Interrupt, 19+timer_, 6, 4+timer_ );
#elif( defined MCF52234 )
      if( timer_ == 0 )
          SetIntc0((long)&Timer0_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 1 )
          SetIntc0((long)&Timer1_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 2 )
          SetIntc0((long)&Timer2_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 3 )
          SetIntc0((long)&Timer3_Interrupt, 19+timer_, 6, 4+timer_ );
#elif( defined MCF5282 || defined MCF5234 )
      if( timer_ == 0 )
          SetIntc(0, (long)&Timer0_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 1 )
          SetIntc(0, (long)&Timer1_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 2 )
          SetIntc(0, (long)&Timer2_Interrupt, 19+timer_, 6, 4+timer_ );
      else if( timer_ == 3 )
          SetIntc(0, (long)&Timer3_Interrupt, 19+timer_, 6, 4+timer_ );
#elif( defined MCF5441X )
      if( timer_ == 0 )
          SetIntc(0, (long)&Timer0_Interrupt, 32+timer_, 6 );
      else if( timer_ == 1 )
          SetIntc(0, (long)&Timer1_Interrupt, 32+timer_, 6 );
      else if( timer_ == 2 )
          SetIntc(0, (long)&Timer2_Interrupt, 32+timer_, 6 );
      else if( timer_ == 3 )
          SetIntc(0, (long)&Timer3_Interrupt, 32+timer_, 6 );
#else
#error PROCESSOR NOT DEFINED
#endif
      firstInit_ = TRUE;
   }
}

double HiResTimer::readTime()
{
   USER_ENTER_CRITICAL();
   DWORD lowTime = readLow();
   DWORD highTime = resetCount_;
   USER_EXIT_CRITICAL();
   double CurrentTime = (
		   (double) TIME_PER_CLK * ( // How long is a clock cycle?
			   (double) lowTime + (  // The count of the current DMA count register
				   (double) highTime * ( // How many times have we reset?
					   (double) simTimer_->trr + 1.0 ) // What is the overflow value?
				   )
			) * (double) (prescaler_ + 1.0));  // The prescaler - how many cycle per DMA tick
   return (CurrentTime);
}

void HiResTimer::stopClear()
{
   simTimer_->tmr &= ~(0x6);
   simTimer_->tcn = 0x0;
   simTimer_->ter = 0x3;
   resetCount_ = 0;
}

void HiResTimer::delay(double delayTime)
{
   if( delayTime == 0 ) return;
   init(delayTime);
   start();
   delayCalled_ = TRUE;
   OSSemPend(&delaySem_, 0);
   return;
}

void HiResTimer::pollingDelay(double delayTime)
{
   stopClear();
   DWORD ReferenceTicks = (delayTime * (double) NB_CPU_CLK );
   simTimer_->tmr = 0x9;
   simTimer_->trr = ReferenceTicks;
   start();
   while ((simTimer_->ter & 0x2) == 0)
      ;

}

void HiResTimer::clockGenerator(double frequency)
{
   double ToggleTime = (double) (1.0 / ( 2 * frequency ));
   init(ToggleTime);
   simTimer_->tmr &= ~(0x10); // Disable interrupts
   simTimer_->tmr |= 0x20;
   start();
}

/*
 * Returns a reference to the given timer
 */
HiResTimer* HiResTimer::getHiResTimer(int timer)
{
    if (( timer >= TIMER_COUNT ) || ( timer < 0 ) )
        return NULL;

    // HACK! Relies on the fact that all RAM is zeroed out at startup
    if (!timerCreated[timer])
    {
    	timers[timer] = HiResTimer(timer);
    	timerCreated[timer] = TRUE;
    	iprintf("created timer: %d", timer);
    }
    return &timers[timer];
}

/*
 * Private constructor for HiResTimers
 */
HiResTimer::HiResTimer(int timer)
{
    timer_ = timer;
    simTimer_ = &(sim.timer[timer]);
    prescaler_ = 0;
    resetCount_ = 0;
    firstInit_ = FALSE;
    delayCalled_ = FALSE;

    interruptFunction_ = NULL;
}

HiResTimer& HiResTimer::operator=( const HiResTimer& from)
{
	timer_ = from.timer_;
	simTimer_ = from.simTimer_;
	prescaler_ = from.prescaler_;
	resetCount_ = from.resetCount_;
	delaySem_ = from.delaySem_;
	firstInit_ = from.firstInit_;
	delayCalled_ = from.delayCalled_;
	interruptFunction_ = from.interruptFunction_;
	return *this;
}

char* HiResTimer::toString()
{
	static char buf[256];
	sprintf(buf, "Timer: %d\r\n"
			"  simTimer:\r\n"
			"    tmr: %X\r\n"
			"    txmr: %X\r\n"
			"    ter: %X\r\n"
			"    trr: %lX\r\n"
			"    tcr: %lX\r\n"
			"    tcn: %lX\r\n"
			"  prescaler: %d\r\n"
			"  resetCount: %ld\r\n"
			"  firstInit: %lX\r\n"
			"  delayCalled: %lX\r\n"
			"  interruptFunction: %lX\r\n", timer_, simTimer_->tmr, simTimer_->txmr, simTimer_->ter, simTimer_->trr, simTimer_->tcr, simTimer_->tcn,
			prescaler_, resetCount_, firstInit_, delayCalled_, (unsigned long)interruptFunction_);
	return buf;
}

