/* $Revision: 1.4 $ */

/******************************************************************************
 *
 * Copyright 2012 NetBurner  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or its derivatives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non-NetBurner Hardware.
 *   Please contact sales@Netburner.com for more information.
 *
 *   NetBurner makes no representation or warranties
 *   with respect to the performance of this computer program, and
 *   specifically disclaims any responsibility for any damages,
 *   special or consequential, connected with the use of this program.
 *
 *---------------------------------------------------------------------
 * NetBurner
 * 5405 Morehouse Dr.
 * San Diego, Ca. 92121
 *
 * information available at:  http://www.netburner.com
 * E-Mail info@netburner.com
 *
 * Support is available: E-Mail support@netburner.com
 *
 *****************************************************************************/


/* Select the card type */
#ifndef _HIRESTIMER_H
#define _HIRESTIMER_H

#if defined MCF5272
#error HiResTimer is not compatible with, nor supported on the MCF5272 processor.
#endif

#include <ucos.h>
#include <stdio.h>
#include <sim.h>

#define LOGME iprintf("We got to line %d in file %s\r\n",__LINE__ , __FILE__ ); OSTimeDly(2);

#define DEFAULT_TIMER 0
#define TIMER_COUNT 4

class HiResTimer{
    static HiResTimer timers[TIMER_COUNT];
    static BYTE timerCreated[TIMER_COUNT];

    int timer_;  // Timer number
    volatile timerstruct *simTimer_; // Timer struct from sim.h
    volatile int prescaler_;    // The value of the hardware prescaler
    volatile DWORD resetCount_; // Holds the count of the timer resets
    OS_SEM delaySem_;  // OS semaphore struct
    DWORD firstInit_;  // Have we set up this timer before ?
    volatile DWORD delayCalled_;   // Is the timer being used for a delay?

    // The method that will be called when the timer triggers an interupt
    void (*interruptFunction_)();

    // Disable construction and copy construction
    HiResTimer() {};
    HiResTimer( const HiResTimer& from) {};
    HiResTimer& operator=( const HiResTimer& from);
    // Only actual constructor
    HiResTimer( int timer );
    ~HiResTimer() {};

    // Fire the object's interrupt method
    void fireInterrupt();

    public:
    	// class function to fire the interrupt function for the given timer
    	// called by the Interrupt macro routine
        static void fireInterrupt( int timer );

        void init(double InterruptTime = 0); // Initializes the timer for use

        /*
         * Starts the selected timer. The user MUST call HiResTimerInit for the given timer
         * before calling this method.
         */
        inline void start()
        {
           // Setting the CLK bits to 01 on the DTMR register, starting the timer
           simTimer_->tmr |= 0x2;
        };

        inline DWORD readHigh() { return resetCount_; }; // Reads the number of timer overflows/resets/interrupts
        inline DWORD readLow() { return simTimer_->tcn; }; // Reads the timer register value
        inline int getPrescaler() { return prescaler_; }; // Returns the value of the prescaler
        double readTime(); // Ouputs the number of seconds since the timer was started

        inline void stop() { simTimer_->tmr &= ~(0x6); };	   // Stops the timer
        void stopClear();  // Stops the timer and clears the registers

        void delay(double DelayTime); 			// Starts a precise delay that will switch tasks while waiting (Do not use for shorter than ~150 useconds)
        void pollingDelay(double DelayTime);	// Starts a precise delay that will busy wait

        void clockGenerator(double Frequency);	// Set up a clock at the chosen frequency

        inline void setInterruptFunction( void (*interruptFunction)() )
			{ interruptFunction_ = interruptFunction; }; // Set the method to be called on interrupt triggers
        inline void clearInterruptFunction() { interruptFunction_ = NULL; };			// Clear the interrupt method

        char* toString();				// Converts the timer object to a printable c string

        // Returns a pointer for the given timer number, returns NULL if the timer does not exist
        static HiResTimer* getHiResTimer( int timer = DEFAULT_TIMER );
};

// Clock speed macros
extern unsigned long CPU_CLOCK;
#if (defined MCF5282 || defined MCF5213)
#define NB_CPU_CLK      ( CPU_CLOCK)
#elif defined MCF5208
#define NB_CPU_CLK      ( CPU_CLOCK / 2 )
#elif defined MCF53017
#define NB_CPU_CLK      ( CPU_CLOCK / 2 )
#elif (defined MCF5234 || defined MCF5270)
#define NB_CPU_CLK      ( CPU_CLOCK / 2 )
#elif (defined MOD5441X)
#define NB_CPU_CLK      ( CPU_CLOCK / 2 )
#elif (defined MCF52234)
#define NB_CPU_CLK      ( CPU_CLOCK )
#else
#error PROCESSOR NOT DEFINED
#endif

#define TIME_PER_CLK (1.0 / NB_CPU_CLK)

#endif   /* _HIRESTIMER_H */

